module MAIN
(
SYSTEM_CLOCK,
RS232_RX_DATA,
RS232_RTS_IN,

LED_0,
LED_1,
LED_2,
LED_3,

VGA_OUT_PIXEL_CLOCK,
VGA_COMP_SYNCH,
VGA_OUT_BLANK_Z,
VGA_HSYNCH,
VGA_VSYNCH,
VGA_OUT_RED,
VGA_OUT_GREEN,
VGA_OUT_BLUE,

RS232_TX_DATA,
RS232_DSR_OUT,
RS232_CTS_OUT 
);

input				SYSTEM_CLOCK;				// 100MHz LVTTL SYSTEM CLOCK
input				RS232_RX_DATA;
input				RS232_RTS_IN;

output			LED_0;						// User LED 0
output			LED_1;						// User LED 1
output			LED_2;						// User LED 2
output			LED_3;						// User LED 3

output 			VGA_OUT_PIXEL_CLOCK;		// pixel clock for the video DAC
output			VGA_COMP_SYNCH;			// composite sync for the video DAC
output 			VGA_OUT_BLANK_Z;			// composite blanking for the video DAC
output 			VGA_HSYNCH;					// horizontal sync for the VGA output connector
output			VGA_VSYNCH;					// vertical sync for the VGA output connector
output [7:0] 	VGA_OUT_RED;				// RED DAC data
output [7:0] 	VGA_OUT_GREEN;				// GREEN DAC data
output [7:0] 	VGA_OUT_BLUE;				// BLUE DAC data

output			RS232_TX_DATA;
output			RS232_DSR_OUT;
output			RS232_CTS_OUT;

wire				system_clock_buffered;	// buffered SYSTEM CLOCK
wire				pixel_clock;				// generated from SYSTEM CLOCK
wire				reset;						// reset asserted when DCMs are NOT LOCKED

wire [7:0]		vga_red_data;				// red video data
wire [7:0]		vga_green_data;			// green video data
wire [7:0]		vga_blue_data;				// blue video data

// internal video timing signals
wire 				h_synch;
wire 				v_synch;
wire 				comp_synch;
wire 				blank;
wire [10:0]		pixel_count;				// bit mapped pixel position within the line
wire [9:0]		line_count;					// bit mapped line number in a frame lines within the frame

wire [7:0]		RX_data;
wire [7:0] 		RX_TEST;
wire 				RX_data_ready;
//wire 				RX_endofpacket;
//wire				RX_idle;

wire [7:0]	spriteID;
wire [15:0]  X;
wire [15:0]  Y;
//wire [15:0]		X;
//wire [7:0]		spriteID;

//extra stuff that needs to be there, but apparently isn't used...
wire RS232_RTS_IN;
wire RS232_DSR_OUT;
wire RS232_CTS_OUT; //CARRIER DETECT, needs to be one to mean that something is there!!

reg testreg2; //test reg, exists temporarily to test RX_endofpacket

wire led0, led1, led2, led3;

assign LED_0 = led0; // Turn LED_0 Off (0 is on)
assign LED_1 = led1; // Turn LED_1 On
assign LED_2 = led2; // Turn LED_2 Off
assign LED_3 = led3; // Turn LED_3 On

wire [7:0] 		TempTX_data;
assign TempTX_data = 8'h2B; //+ symbol
assign RS232_DSR_OUT = 1;
assign RS232_CTS_OUT = 1;

wire 				CHANGE_STATE;

initial
begin
	testreg2 <= 1;
end

always @(posedge RS232_RX_DATA) begin
	testreg2 <= 0;
end

// instantiate the clock generation module
CLOCK_GEN CLOCK_GEN 
(
SYSTEM_CLOCK,
system_clock_buffered,
pixel_clock,
reset
);

// instantiate the video timing generator
SVGA_TIMING_GENERATION SVGA_TIMING_GENERATION
(
pixel_clock,
reset,
h_synch,
v_synch,
comp_synch,
blank,
pixel_count,
line_count
);

UART UART
(
system_clock_buffered,
RS232_RX_DATA, //incoming data bit signal
RS232_TX_DATA, //outgoing data bit signal.

RX_data,
RX_data, //TX_data, right now just echoing what is received.

RX_data_ready
);


TEST_RX TEST_RX
(
system_clock_buffered,

RX_TEST, //REMEMBER, RX_TEST and RX_data are DIFFERENT!!!
CHANGE_STATE
);


S_RECEIVER S_RECEIVER
(
RX_TEST,
CHANGE_STATE, //CHANGE_STATE,

X,
Y,
spriteID,
draw
);

/*
TestXYSprite test(
system_clock_buffered,

X,
Y,
spriteID,
draw
);
*/

// instantiate the color bar generator
SCREEN_BUFFER SCREEN_BUFFER //COLOR_BARS COLOR_BARS
(
pixel_clock,
pixel_count,
line_count,

X,
Y,
spriteID,
draw,

vga_red_data,
vga_green_data,
vga_blue_data
);

// instantiate the video output mux
VIDEO_OUT VIDEO_OUT
(
pixel_clock,
reset,
vga_red_data,
vga_green_data,
vga_blue_data,
h_synch,
v_synch,
comp_synch,
blank,

VGA_OUT_PIXEL_CLOCK,
VGA_HSYNCH,
VGA_VSYNCH,
VGA_COMP_SYNCH,
VGA_OUT_BLANK_Z,
VGA_OUT_RED,
VGA_OUT_GREEN,
VGA_OUT_BLUE
);

endmodule // MAIN

/*
async_receiver async_receiver
(
system_clock_buffered,
RS232_RX_DATA,

RX_data_ready,
RX_data,
RX_endofpacket,
RS232_CTS_OUT //RX_idle
);

async_transmitter async_transmitter
(
system_clock_buffered, 
RS232_RTS_IN, 
TempTX_data, 

RS232_TX_DATA, 
RS232_DSR_OUT	//needs to be inverted? Should this be DSR_OUT? 
);
*/